-------------------------------------------------------------------------------
-- $Id$
-------------------------------------------------------------------------------
-- fsl_dct32_tb.vhd - Entity and architecture
--
--
-------------------------------------------------------------------------------
-- Filename:        fsl_dct32_tb.vhd
--
-- Description:     
--                  
-- VHDL-Standard:   VHDL'93
-------------------------------------------------------------------------------
-- Structure:   
--              fsl_dct32_tb.vhd
--
-------------------------------------------------------------------------------
-- Author:          
-- Revision:        $Revision$
-- Date:            $Date$
--
-- History:
--     2003-11-12    First Version
--
-------------------------------------------------------------------------------
-- Naming Conventions:
--      active low signals:                     "*_n"
--      clock signals:                          "clk", "clk_div#", "clk_#x" 
--      reset signals:                          "rst", "rst_n" 
--      generics:                               "C_*" 
--      user defined types:                     "*_TYPE" 
--      state machine next state:               "*_ns" 
--      state machine current state:            "*_cs" 
--      combinatorial signals:                  "*_com" 
--      pipelined or register delay signals:    "*_d#" 
--      counter signals:                        "*cnt*"
--      clock enable signals:                   "*_ce" 
--      internal version of output port         "*_i"
--      device pins:                            "*_pin" 
--      ports:                                  - Names begin with Uppercase 
--      processes:                              "*_PROCESS" 
--      component instantiations:               "<ENTITY_>I_<#|FUNC>
-------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

-------------------------------------------------------------------------------

entity fsl_dct32_tb is

end entity fsl_dct32_tb;

-------------------------------------------------------------------------------

architecture VHDL_TEST of fsl_dct32_tb is

  component fsl_dct32 is
    port (
      Clk           : in  std_logic;
      Reset         : in  std_logic;
      FSL_S_Clk     : out std_logic;
      FSL_S_Read    : out std_logic;
      FSL_S_Data    : in  std_logic_vector(0 to 31);
      FSL_S_Control : in  std_logic;
      FSL_S_Exists  : in  std_logic;

      FSL_M_Clk     : out std_logic;
      FSL_M_Write   : out std_logic;
      FSL_M_Data    : out std_logic_vector(0 to 31);
      FSL_M_Control : out std_logic;
      FSL_M_Full    : in  std_logic);
  end component fsl_dct32;

  -- component ports
  signal Reset         : std_logic;                  -- [in]
  signal FSL_S_Clk     : std_logic;                  -- [out]
  signal FSL_S_Read    : std_logic;                  -- [out]
  signal FSL_S_Data    : std_logic_vector(0 to 31);  -- [in]
  signal FSL_S_Control : std_logic;                  -- [in]
  signal FSL_S_Exists  : std_logic;                  -- [in]

  signal FSL_M_Clk     : std_logic;                  -- [out]
  signal FSL_M_Write   : std_logic;                  -- [out]
  signal FSL_M_Data    : std_logic_vector(0 to 31);  -- [out]
  signal FSL_M_Control : std_logic;                  -- [out]
  signal FSL_M_Full    : std_logic;                  -- [in]

  -- clock
  signal Clk : std_logic := '1';

  type std_logic_vector_array is array (natural range 0 to 31) of std_logic_vector(0 to 31);
  type int_array is array (natural range 0 to 31) of integer;
  constant in_value : int_array := (
    936664, 26580, 242025, -78290, -82141, 173722, 228648, 87423, 27669,
    -100558, 7141, 15702, 34876, -11422, -10319, -21307, -5523, 9453, 7433, 1314, 737, -1155, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0);

  signal result : std_logic_vector_array;
  
begin  -- architecture VHDL_TEST

  -- component instantiation
  DUT : fsl_dct32
    port map (
      Clk           => Clk,             -- [in  std_logic]
      Reset         => Reset,           -- [in  std_logic]
      FSL_S_Clk     => FSL_S_Clk,       -- [out std_logic]
      FSL_S_Read    => FSL_S_Read,      -- [out std_logic]
      FSL_S_Data    => FSL_S_Data,      -- [in  std_logic_vector(0 to 31)]
      FSL_S_Control => FSL_S_Control,   -- [in  std_logic]
      FSL_S_Exists  => FSL_S_Exists,    -- [in  std_logic]

      FSL_M_Clk     => FSL_M_Clk,       -- [out std_logic]
      FSL_M_Write   => FSL_M_Write,     -- [out std_logic]
      FSL_M_Data    => FSL_M_Data,      -- [out std_logic_vector(0 to 31)]
      FSL_M_Control => FSL_M_Control,   -- [out std_logic]
      FSL_M_Full    => FSL_M_Full);     -- [in std_logic]

  -- clock generation
  Clk <= not Clk after 10 ns;

  reset <= '1',
           '0' after 20 ns;

  -- waveform generation
  WaveGen_Proc : process
    variable count : integer;
  begin
    -- insert signal assignments here
    FSL_S_Exists  <= '0';
    FSL_S_Control <= '0';
    FSL_M_Full    <= '0';
    count         := 0;
    FSL_S_Data    <= std_logic_vector(to_signed(in_value(count), 32));
    wait until Reset = '0';
    FSL_S_Exists  <= '1';
    for i in 1 to 31 loop
      wait until Clk = '1' and clk'event and FSL_S_Read = '1';
      FSL_S_Data <= std_logic_vector(to_signed(in_value(i), 32));
    end loop;  -- i
    wait until Clk = '1' and clk'event and FSL_S_Read = '1';
    FSL_S_Exists <= '0';

    count := 0;
    for i in 0 to 16 loop
      wait until Clk = '0' and clk'event and FSL_M_Write = '1';
      result(count) <= FSL_M_Data;
      count         := count + 1;
    end loop;  -- i
    wait until Clk = '1' and clk'event;
    FSL_M_Full <= '1';

    for i in 0 to 14 loop
      
      FSL_M_Full <= '1';
      wait until Clk = '1' and clk'event;
      wait until Clk = '1' and clk'event;
      FSL_M_Full <= '0';

      wait until Clk = '0' and clk'event and FSL_M_Write = '1';
      result(count) <= FSL_M_Data;
      count         := count + 1;
      wait until Clk = '1' and clk'event;
    end loop;  -- i

    wait until false;
  end process WaveGen_Proc;

  

end architecture VHDL_TEST;

-------------------------------------------------------------------------------

configuration fsl_dct32_tb_VHDL_TEST_cfg of fsl_dct32_tb is
  for VHDL_TEST
  end for;
end fsl_dct32_tb_VHDL_TEST_cfg;

-------------------------------------------------------------------------------
