-------------------------------------------------------------------------------
-- $Id$
-------------------------------------------------------------------------------
-- fsl_mp3_mul_tb.vhd - Entity and architecture
--
--
-------------------------------------------------------------------------------
-- Filename:        fsl_mp3_mul_tb.vhd
--
-- Description:     
--                  
-- VHDL-Standard:   VHDL'93
-------------------------------------------------------------------------------
-- Structure:   
--              fsl_mp3_mul_tb.vhd
--
-------------------------------------------------------------------------------
-- Author:          
-- Revision:        $Revision$
-- Date:            $Date$
--
-- History:
--     2003-11-12    First Version
--
-------------------------------------------------------------------------------
-- Naming Conventions:
--      active low signals:                     "*_n"
--      clock signals:                          "clk", "clk_div#", "clk_#x" 
--      reset signals:                          "rst", "rst_n" 
--      generics:                               "C_*" 
--      user defined types:                     "*_TYPE" 
--      state machine next state:               "*_ns" 
--      state machine current state:            "*_cs" 
--      combinatorial signals:                  "*_com" 
--      pipelined or register delay signals:    "*_d#" 
--      counter signals:                        "*cnt*"
--      clock enable signals:                   "*_ce" 
--      internal version of output port         "*_i"
--      device pins:                            "*_pin" 
--      ports:                                  - Names begin with Uppercase 
--      processes:                              "*_PROCESS" 
--      component instantiations:               "<ENTITY_>I_<#|FUNC>
-------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

-------------------------------------------------------------------------------

entity fsl_mp3_mul_tb is

end entity fsl_mp3_mul_tb;

-------------------------------------------------------------------------------
library fsl_mp3_mul_v1_00_a;
use fsl_mp3_mul_v1_00_a.all;

architecture VHDL_TEST of fsl_mp3_mul_tb is

  component fsl_mp3_mul is
    port (
      Clk           : in  std_logic;
      Reset         : in  std_logic;
      FSL_S_Clk     : out std_logic;
      FSL_S_Read    : out std_logic;
      FSL_S_Data    : in  std_logic_vector(0 to 31);
      FSL_S_Control : in  std_logic;
      FSL_S_Exists  : in  std_logic;

      FSL_M_Clk     : out std_logic;
      FSL_M_Write   : out std_logic;
      FSL_M_Data    : out std_logic_vector(0 to 31);
      FSL_M_Control : out std_logic;
      FSL_M_Full    : in  std_logic);
  end component fsl_mp3_mul;

  -- component ports
  signal Reset         : std_logic;                  -- [in]
  signal FSL_S_Clk     : std_logic;                  -- [out]
  signal FSL_S_Read    : std_logic;                  -- [out]
  signal FSL_S_Data    : std_logic_vector(0 to 31);  -- [in]
  signal FSL_S_Control : std_logic;                  -- [in]
  signal FSL_S_Exists  : std_logic;                  -- [in]

  signal FSL_M_Clk     : std_logic;                  -- [out]
  signal FSL_M_Write   : std_logic;                  -- [out]
  signal FSL_M_Data    : std_logic_vector(0 to 31);  -- [out]
  signal FSL_M_Control : std_logic;                  -- [out]
  signal FSL_M_Full    : std_logic;                  -- [in]

  -- clock
  signal Clk : std_logic := '1';

  type std_logic_vector_array is array (natural range 0 to 35) of std_logic_vector(0 to 31);
  type int_array is array (natural range 0 to 17) of integer;
  constant in1_value : int_array := (0 , -4560891, 545094, 3145727, 7899091,
                                     -3736701, -2965821, -5219611, 5284491, -4194303,
                                     13655, -2082053, -6822472, -3719953, 7194006,
                                     2219008, -152833, 5889660);       

  constant in2_value : int_array := (6634075 , 1, 545094, 3145727, 7899091,
                                     -3736701, -2965821, -5219611, 5284492, -4194303,
                                     13655, -2082053, -6822472, -3719953, 7194006,
                                     2219008, -152833, 5889660);       

  signal result : std_logic_vector_array;
  
begin  -- architecture VHDL_TEST

  -- component instantiation
  DUT : fsl_mp3_mul
    port map (
      Clk           => Clk,             -- [in  std_logic]
      Reset         => Reset,           -- [in  std_logic]
      FSL_S_Clk     => FSL_S_Clk,       -- [out std_logic]
      FSL_S_Read    => FSL_S_Read,      -- [out std_logic]
      FSL_S_Data    => FSL_S_Data,      -- [in  std_logic_vector(0 to 31)]
      FSL_S_Control => FSL_S_Control,   -- [in  std_logic]
      FSL_S_Exists  => FSL_S_Exists,    -- [in  std_logic]

      FSL_M_Clk     => FSL_M_Clk,       -- [out std_logic]
      FSL_M_Write   => FSL_M_Write,     -- [out std_logic]
      FSL_M_Data    => FSL_M_Data,      -- [out std_logic_vector(0 to 31)]
      FSL_M_Control => FSL_M_Control,   -- [out std_logic]
      FSL_M_Full    => FSL_M_Full);     -- [in std_logic]

  -- clock generation
  Clk <= not Clk after 10 ns;

  reset <= '1',
           '0' after 20 ns;

  -- waveform generation
  WaveGen_Proc : process
    variable count : integer;
  begin
    -- insert signal assignments here
    FSL_S_Exists  <= '0';
    FSL_S_Control <= '0';
    FSL_M_Full    <= '0';
    count         := 0;
    FSL_S_Data    <= std_logic_vector(to_signed(in1_value(count), 32));
    wait until Reset = '0';
      
    count         := 0;
    for i in 0 to 7 loop

      FSL_S_Data    <= std_logic_vector(to_signed(in1_value(i), 32));
      FSL_S_Exists <= '1';
      wait until Clk = '1' and clk'event and FSL_S_Read = '1'; 
      FSL_S_Exists <= '0';

      wait until Clk = '1' and clk'event;
      wait until Clk = '1' and clk'event;
      wait until Clk = '1' and clk'event;

      FSL_S_Exists <= '1';
      FSL_S_Data <= std_logic_vector(to_signed(in2_value(i), 32));
      wait until Clk = '1' and clk'event and FSL_S_Read = '1'; 
      FSL_S_Exists <= '0';
      
      wait until Clk = '1' and clk'event;
      wait until Clk = '1' and clk'event;
      wait until Clk = '1' and clk'event;
      wait until Clk = '1' and clk'event;
    end loop;  -- i

    FSL_M_Full <= '1','0' after 202 ns;

    FSL_S_Data    <= std_logic_vector(to_signed(in1_value(8), 32));
    FSL_S_Exists <= '1';
    wait until Clk = '1' and clk'event and FSL_S_Read = '1'; 

    FSL_S_Exists <= '1';
    FSL_S_Data <= std_logic_vector(to_signed(in2_value(8), 32));
    wait until Clk = '1' and clk'event and FSL_S_Read = '1'; 
    FSL_S_Exists <= '0';

    wait until false;
  end process WaveGen_Proc;

  

end architecture VHDL_TEST;

-------------------------------------------------------------------------------

configuration fsl_mp3_mul_tb_VHDL_TEST_cfg of fsl_mp3_mul_tb is
  for VHDL_TEST
  end for;
end fsl_mp3_mul_tb_VHDL_TEST_cfg;

-------------------------------------------------------------------------------
