
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.numeric_std.all;

entity fsl_tb is
end fsl_tb;

library fsl_v20_v1_00_b;
use  fsl_v20_v1_00_b.all;

architecture tb of fsl_tb is

component fsl_wrapper is
  port (
    -- Clock and reset signals
    FSL_Clk : in  std_logic;
    SYS_Rst : in  std_logic;
    FSL_Rst : out std_logic;

    -- FSL master signals
    FSL_M_Clk     : in  std_logic;
    FSL_M_Data    : in  std_logic_vector(0 to 31);
    FSL_M_Control : in  std_logic;
    FSL_M_Write   : in  std_logic;
    FSL_M_Full    : out std_logic;

    -- FSL slave signals
    FSL_S_Clk     : in  std_logic;
    FSL_S_Data    : out std_logic_vector(0 to 31);
    FSL_S_Control : out std_logic;
    FSL_S_Read    : in  std_logic;
    FSL_S_Exists  : out std_logic 
    );
end component;

  signal SYS_Rst         : std_logic;                  -- [in]
  signal FSL_S_Clk     : std_logic;                  -- [out]
  signal FSL_S_Read    : std_logic;                  -- [out]
  signal FSL_S_Data    : std_logic_vector(0 to 31);  -- [in]
  signal FSL_S_Control : std_logic;                  -- [in]
  signal FSL_S_Exists  : std_logic;                  -- [in]

  signal FSL_M_Clk     : std_logic;                  -- [out]
  signal FSL_M_Write   : std_logic;                  -- [out]
  signal FSL_M_Data    : std_logic_vector(0 to 31);  -- [out]
  signal FSL_M_Control : std_logic;                  -- [out]
  signal FSL_M_Full    : std_logic;                  -- [in]

  -- clock
  signal FSL_Clk : std_logic := '1';

begin
  
  DUT: fsl_wrapper
    port map (
      FSL_Clk       => FSL_Clk,             -- [in  std_logic]
      SYS_Rst       => SYS_Rst,           -- [in  std_logic]
      FSL_S_Clk     => FSL_S_Clk,       -- [out std_logic]
      FSL_S_Read    => FSL_S_Read,      -- [out std_logic]
      FSL_S_Data    => FSL_S_Data,      -- [in  std_logic_vector(0 to 31)]
      FSL_S_Control => FSL_S_Control,   -- [in  std_logic]
      FSL_S_Exists  => FSL_S_Exists,    -- [in  std_logic]

      FSL_M_Clk     => FSL_M_Clk,       -- [out std_logic]
      FSL_M_Write   => FSL_M_Write,     -- [out std_logic]
      FSL_M_Data    => FSL_M_Data,      -- [out std_logic_vector(0 to 31)]
      FSL_M_Control => FSL_M_Control,   -- [out std_logic]
      FSL_M_Full    => FSL_M_Full);     -- [in std_logic]

  -- clock generation
  FSL_Clk <= not FSL_Clk after 10 ns;

  SYS_Rst <= '1',
	     '0' after 20 ns;


  -- waveform generation
  WaveGen_Proc : process
    variable count : integer;
  begin

    for i in 0 to 16 loop
      wait until FSL_Clk = '1' and FSL_Clk'event;
    end loop;

    count         := 3;
    FSL_M_Control <= '1';

    FSL_M_Write   <= '1', '0' after 5120 ns;
    FSL_S_Read    <= '0', '1' after 5600 ns;

    FSL_M_Data    <= std_logic_vector(to_unsigned(count, 32));
    wait until SYS_Rst = '0';
    for i in 0 to 260 loop
      count := count + 1;
      wait until FSL_Clk = '1' and FSL_Clk'event;
    end loop;
    
    wait until FSL_Clk = '1' and FSL_Clk'event;

    for i in 0 to 260 loop
      FSL_M_Write  <= '0';
      FSL_S_Read <= '1';
      wait until FSL_Clk = '1' and FSL_Clk'event;
    end loop;

  end process WaveGen_Proc;

end architecture tb;

configuration fsl_conf of fsl_tb is
  for tb
  end for;
end fsl_conf;
